<?php
/**
 * @Package Paystar Payment Module For Boxbiling
 * ------------------------------
 * @Author : Shayan Kiyani
 * @Website : https://paystar.ir
 * @Mail : info[at]paystar.ir
 * ------------------------------
 * @Version 1.0
 * @Copyright 2016-2019 paystar.ir
 */
class Payment_Adapter_paystar extends Payment_AdapterAbstract
{
	public $go_url;
	
    public function init()
    {
        if(!extension_loaded('curl'))
            throw new Payment_Exception('Curl extension is not enabled'); 
        if(!$this->getParam('pin'))
        	throw new Payment_Exception('شناسه درگاه نامعتبر است .');
    }

    public static function getConfig()
    {
        return array(
            'supports_one_time_payments'   =>  true,
            'supports_subscriptions'     =>  false,
            'description'     =>  'این پلاگین توسط <a href="https://paystar.ir" target="_blank"> شایان کیانی </a> طراحی شده است . <br>خواهشمندیم کپی نفرمایید .',
            'form'  => array(
                'pin' => array('text', array(
                            'label' => 'شناسه درگاه پرداخت شما',
                            'validators'=>array('noempty'),
                    ),
                ),
                'bank' => array('select', array(
                            'multiOptions' => Payment_Adapter_paystar::ws_config()['bank'],
                            'label' => 'بانک پیشفرض برای انجام تراکنش ها',
                    ),
                 ),					
            ),
        );
    }
	
    public function getType()
    {
        return Payment_AdapterAbstract::TYPE_FORM;
    }
	
    public function getServiceUrl()
    {
        if($this->testMode) 
            throw new Payment_Exception('درگاه مورد نظر متسفانه حالت آزمایشی را پشتیبانی نمیکند .');
        
		return $this->go_url;
    }
	
	public function singlePayment(Payment_Invoice $invoice)
	{
		$b = $invoice->getBuyer();	
		$email = $b->getEmail();
		$phone = $b->getPhone();
	
		$data = array();
		
		if($invoice->getCurrency() != 'IRR')
			throw new Payment_Exception('خطای واحد پولی');
		else
		{
			try 
			{
				$description = "شماره سفارش خریدار : {$invoice->getId()}";
				$description .= "\nایمیل خریدار : {$email}";
				$description .= !empty($phone) ? "\nتلفن خریدار : {$phone}" : '';

				$f = array(
					'amount' => $invoice->getTotalWithTax()/10,
					'pin' => trim($this->getParam('pin')),
					'callback' => $this->getParam('redirect_url'),
					'bank' => $this->getParam('bank'),
					'description' => $description
				);

				$f_ = '';
				foreach($f as $k=>$v) 
					$f_ .= $k . '=' . urlencode($v) . '&'; 
					
				rtrim($f_, '&');

				$ch = curl_init();
				curl_setopt($ch,CURLOPT_URL, "https://{$this->ws_config()['address']}/api/create/");
				curl_setopt($ch,CURLOPT_POST, count($f));
				curl_setopt($ch,CURLOPT_POSTFIELDS, $f_);
				curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
				$res = curl_exec($ch);
				curl_close($ch);
				if (!$res) 
					throw new Payment_Exception('اتصال به بانک برقرار نشد .');
				else 
				{
					if(strlen($res) > 5)
						$this->go_url = 'https://' . $this->ws_config()['address'] . '/paying/' . $res;	
					else
						throw new Payment_Exception($res);				
				}
			}
			catch(Exception $e) 
			{
				throw new Payment_Exception('اتصال به بانک برقرار نشد .');
			} 	
		}
	}

	public function recurrentPayment(Payment_Invoice $invoice)
	{
		throw new Payment_Exception('Not implemented yet');
	}
	
    public function processTransaction($api_admin, $id, $data, $gateway_id)
    {
		if(!empty($data['get']['bb_invoice_id']))
		{
			$invoice = $api_admin->invoice_get(array('id'=>$data['get']['bb_invoice_id']));	
			$transid = $data['post']['transid'];
			if(!empty($transid))
			{
				$res = false;
				try
				{
					$f = array(
						'amount' => $invoice['total']/10,
						'pin' => trim($this->getParam('pin')),
						'transid' => $transid,
					);
					
					$f_ = '';
					foreach($f as $k=>$v) 
						$f_ .= $k . '=' . urlencode($v) . '&'; 
						
					rtrim($f_, '&');

					$ch = curl_init();
					curl_setopt($ch,CURLOPT_URL, "https://{$this->ws_config()['address']}/api/verify/");
					curl_setopt($ch,CURLOPT_POST, count($f));
					curl_setopt($ch,CURLOPT_POSTFIELDS, $f_);
					curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
					$res = curl_exec($ch);
					curl_close($ch);
				}
				catch(Exception $e) 
				{
					throw new Payment_Exception('اتصال به بانک برقرار نشد .');
				}
			}
			else
				throw new Payment_Exception('پارامترهای ارسالی صحیح نمیباشد .');

			if($res == 1) 
			{ 
				$tx = $api_admin->invoice_transaction_get(array('id'=>$id));				
				if($tx['txn_status'] != 'complete')
				{
					$d = array(
						'id' => $id,
						'invoice_id' => $data['get']['bb_invoice_id'],
						'currency' => $invoice['currency'],
						'txn_status' => 'complete',
						'txn_id' => $transid,
						'amount' => $invoice['total'],
						'type' => 'payment',
						'status' => Payment_Transaction::STATUS_COMPLETE,
						'updated_at' => date('c'),
					);
					$api_admin->invoice_transaction_update($d);
					$bd = array(
						'id'            =>  $invoice['client']['id'],
						'amount'        =>  $invoice['total'],
						'description'   =>  'transaction id : ' . $data['get']['bb_invoice_id'] . ' - Bank transaction id : ' . $transid,
						'type'          =>  'paystar',
						'rel_id'        =>  $data['get']['bb_invoice_id'],
					);

					$api_admin->client_balance_add_funds($bd);
					$api_admin->invoice_update(array('id'=>$invoice['id'], 'status'=>'paid'));
					$api_admin->invoice_batch_pay_with_credits(array('client_id'=>$invoice['client']['id']));					
				}
			}
			else
				throw new Payment_Exception($res);	
		}	
		else
			throw new Payment_Exception('پارامتر های ارسالی صحیح نمیباشد .');		
    }

    public function isIpnValid($data, Payment_Invoice $invoice)
    {
        $ipn = $data['post'];
        return true;
    }
	
    private function ws_config()
    {
		require(dirname(__FILE__) . '/../../../paystarconfig.php');
		return $ws_config;
    }
}